local TIMER_NAME = "GOAT_REMILKABLE"

local function timerdone(inst, data)
	if not data or data.name ~= TIMER_NAME then return end 
	if inst.components.goat_milkable then 
		inst.components.goat_milkable.milkable = true 
	end 
end

local function init(inst)
	if inst.components.timer:TimerExists(TIMER_NAME) then return end 
	local time = inst.components.goat_milkable:GetTime(true)
	inst.components.timer:StartTimer(TIMER_NAME, time) 
end 

local goat_milkable = Class(function(self, inst)
	self.inst = inst 
	self.milkable = false 
	self.time = TUNING.GOAT_REMILKABLE_TIME
	
	if not inst.components.timer then inst:AddComponent"timer" end 
	inst:ListenForEvent("timerdone", timerdone)
	inst:DoTaskInTime(0, init)
end)

function goat_milkable:CanMilk(doer)
	if self.inst.components.sleeper and self.inst.components.sleeper:IsAsleep() then 
		return true 
	end 
end 

function goat_milkable:Milk(doer)
	if not self.milkable then 
		local manager = GetSeasonManager()
		
		if manager:IsRaining() and math.random() < 0.5 then 
			manager:DoLightningStrike(self.inst:GetPosition())
		end 
		
		self.inst:PushEvent("attacked", {attacker = doer, damage = 0})
		
	elseif doer.components.inventory then  
		local milk = SpawnPrefab"goatmilk"
		doer.components.inventory:GiveItem(milk)
		
		self.milkable = false 
		local time = self:GetTime()
		self.inst.components.timer:StartTimer(TIMER_NAME, time)
	end 
	
	return true 
end 

function goat_milkable:GetTime(init)
	local time = self.time * (0.75 + math.random() * 0.5)
	if init then time = time * 0.5 end 
	return time 
end 

function goat_milkable:OnSave()
	return {
		milkable = self.milkable
	}
end 

function goat_milkable:OnLoad(data)
	if data and data.milkable ~= nil then 
		self.milkable = data.milkable
	end 
end 

return goat_milkable